@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.h 4.15.1.1 2019/06/16 09:13:45 kls Exp $
+ * $Id: config.h 5.8 2021/12/27 13:31:29 kls Exp $
  */
 
 #ifndef __CONFIG_H
@@ -22,13 +22,13 @@
 
 // VDR's own version number:
 
-#define VDRVERSION  "2.4.1"
-#define VDRVERSNUM   20401  // Version * 10000 + Major * 100 + Minor
+#define VDRVERSION  "2.6.0"
+#define VDRVERSNUM   20600  // Version * 10000 + Major * 100 + Minor
 
 // The plugin API's version number:
 
-#define APIVERSION  "2.4.1"
-#define APIVERSNUM   20401  // Version * 10000 + Major * 100 + Minor
+#define APIVERSION  "2.6.0"
+#define APIVERSNUM   20600  // Version * 10000 + Major * 100 + Minor
 
 // When loading plugins, VDR searches them by their APIVERSION, which
 // may be smaller than VDRVERSION in case there have been no changes to
@@ -46,6 +46,13 @@
 
 #define TIMERMACRO_TITLE    "TITLE"
 #define TIMERMACRO_EPISODE  "EPISODE"
+#define TIMERMACRO_BEFORE   "{<}"
+#define TIMERMACRO_MATCH    "{=}"
+#define TIMERMACRO_AFTER    "{>}"
+
+#define TIMERPATTERN_AVOID  "@"
+#define TIMERPATTERN_BEGIN  "^"
+#define TIMERPATTERN_END    "$"
 
 #define MINOSDWIDTH   480
 #define MAXOSDWIDTH  1920
