@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.c 4.27.1.3 2019/05/28 15:55:44 kls Exp $
+ * $Id: device.c 5.3 2021/06/08 14:57:26 kls Exp $
  */
 
 #include "device.h"
@@ -58,6 +58,11 @@ bool cDeviceHook::DeviceProvidesTransponder(const cDevice *Device, const cChanne
   return true;
 }
 
+bool cDeviceHook::DeviceProvidesEIT(const cDevice *Device) const
+{
+  return true;
+}
+
 // --- cDevice ---------------------------------------------------------------
 
 // The minimum number of unknown PS1 packets to consider this a "pre 1.3.19 private stream":
@@ -299,7 +304,7 @@ cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool LiveView
              imp <<= 1; imp |= LiveView ? !device[i]->IsPrimaryDevice() || ndr : 0;                                  // prefer the primary device for live viewing if we don't need to detach existing receivers
              imp <<= 1; imp |= !device[i]->Receiving() && (device[i] != cTransferControl::ReceiverDevice() || device[i]->IsPrimaryDevice()) || ndr; // use receiving devices if we don't need to detach existing receivers, but avoid primary device in local transfer mode
              imp <<= 1; imp |= device[i]->Receiving();                                                               // avoid devices that are receiving
-             imp <<= 4; imp |= GetClippedNumProvidedSystems(4, device[i]) - 1;                                       // avoid cards which support multiple delivery systems
+             imp <<= 5; imp |= GetClippedNumProvidedSystems(5, device[i]) - 1;                                       // avoid cards which support multiple delivery systems
              imp <<= 1; imp |= device[i] == cTransferControl::ReceiverDevice();                                      // avoid the Transfer Mode receiver device
              imp <<= 8; imp |= device[i]->Priority() - IDLEPRIORITY;                                                 // use the device with the lowest priority (- IDLEPRIORITY to assure that values -100..99 can be used)
              imp <<= 8; imp |= ((NumUsableSlots && !HasInternalCam) ? SlotPriority[j] : IDLEPRIORITY) - IDLEPRIORITY;// use the CAM slot with the lowest priority (- IDLEPRIORITY to assure that values -100..99 can be used)
@@ -421,8 +426,8 @@ cDevice *cDevice::GetDeviceForTransponder(const cChannel *Channel, int Priority)
             return d; // if any device is tuned to the transponder, we're done
          if (d->ProvidesTransponder(Channel)) {
             if (d->MaySwitchTransponder(Channel))
-               Device = d; // this device may switch to the transponder without disturbing any receiver or live view
-            else if (!d->Occupied() && d->MaySwitchTransponder(Channel)) { // MaySwitchTransponder() implicitly calls Occupied()
+               return d; // this device may switch to the transponder without disturbing any receiver or live view
+            else if (!d->Occupied() && !d->IsBonded()) { // MaySwitchTransponder() implicitly calls Occupied()
                if (d->Priority() < Priority && (!Device || d->Priority() < Device->Priority()))
                   Device = d; // use this one only if no other with less impact can be found
                }
@@ -718,6 +723,17 @@ bool cDevice::DeviceHooksProvidesTransponder(const cChannel *Channel) const
   return true;
 }
 
+bool cDevice::DeviceHooksProvidesEIT(void) const
+{
+  cDeviceHook *Hook = deviceHooks.First();
+  while (Hook) {
+        if (!Hook->DeviceProvidesEIT(this))
+           return false;
+        Hook = deviceHooks.Next(Hook);
+        }
+  return true;
+}
+
 bool cDevice::ProvidesTransponder(const cChannel *Channel) const
 {
   return false;
@@ -787,14 +803,14 @@ bool cDevice::SwitchChannel(const cChannel *Channel, bool LiveView)
   if (LiveView) {
      isyslog("switching to channel %d %s (%s)", Channel->Number(), *Channel->GetChannelID().ToString(), Channel->Name());
      cControl::Shutdown(); // prevents old channel from being shown too long if GetDevice() takes longer
-                           // and, if decrypted, this removes the now superflous PIDs from the CAM, too
+                           // and, if decrypted, this removes the now superfluous PIDs from the CAM, too
      }
   for (int i = 3; i--;) {
       switch (SetChannel(Channel, LiveView)) {
         case scrOk:           return true;
-        case scrNotAvailable: Skins.Message(mtInfo, tr("Channel not available!"));
+        case scrNotAvailable: Skins.QueueMessage(mtInfo, tr("Channel not available!"));
                               return false;
-        case scrNoTransfer:   Skins.Message(mtError, tr("Can't start Transfer Mode!"));
+        case scrNoTransfer:   Skins.QueueMessage(mtError, tr("Can't start Transfer Mode!"));
                               return false;
         case scrFailed:       break; // loop will retry
         default:              esyslog("ERROR: invalid return value from SetChannel");
@@ -810,7 +826,7 @@ bool cDevice::SwitchChannel(int Direction)
   Direction = sgn(Direction);
   if (Direction) {
      cControl::Shutdown(); // prevents old channel from being shown too long if GetDevice() takes longer
-                           // and, if decrypted, this removes the now superflous PIDs from the CAM, too
+                           // and, if decrypted, this removes the now superfluous PIDs from the CAM, too
      int n = CurrentChannel() + Direction;
      int first = n;
      LOCK_CHANNELS_READ;
@@ -842,6 +858,13 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
   cStatus::MsgChannelSwitch(this, 0, LiveView);
 
   if (LiveView) {
+     if (IsPrimaryDevice() && !Replaying() && !Transferring()) { // this is only for FF DVB cards!
+        LOCK_CHANNELS_READ;
+        if (const cChannel *ch = Channels->GetByNumber(currentChannel)) {
+           if (patFilter)
+              patFilter->Release(ch->Sid());
+           }
+        }
      StopReplay();
      DELETENULL(liveSubtitle);
      DELETENULL(dvbSubtitleConverter);
@@ -882,8 +905,6 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
      if (SetChannelDevice(Channel, LiveView)) {
         // Start section handling:
         if (sectionHandler) {
-           if (patFilter)
-              patFilter->Trigger(Channel->Sid());
            sectionHandler->SetChannel(Channel);
            sectionHandler->SetStatus(true);
            }
@@ -897,6 +918,8 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
 
   if (Result == scrOk) {
      if (LiveView && IsPrimaryDevice()) {
+        if (patFilter) // this is only for FF DVB cards!
+           patFilter->Request(Channel->Sid());
         currentChannel = Channel->Number();
         // Set the available audio tracks:
         ClrAvailableTracks();
@@ -1786,11 +1809,13 @@ bool cDevice::AttachReceiver(cReceiver *Receiver)
                Receiver->scramblingTimeout = TS_SCRAMBLING_TIMEOUT;
                bool KnownToDecrypt = ChannelCamRelations.CamDecrypt(Receiver->ChannelID(), camSlot->MasterSlotNumber());
                if (KnownToDecrypt)
-                  Receiver->scramblingTimeout *= 10; // give it time to receive ECM/EMM
+                  Receiver->scramblingTimeout *= 9; // give it time to receive ECM/EMM (must be less than MAXBROKENTIMEOUT in recorder.c!)
                if (Receiver->ChannelID().Valid())
                   dsyslog("CAM %d: %sknown to decrypt channel %s (scramblingTimeout = %ds)", camSlot->MasterSlotNumber(), KnownToDecrypt ? "" : "not ", *Receiver->ChannelID().ToString(), Receiver->scramblingTimeout);
                }
             }
+         if (patFilter && Receiver->ChannelID().Valid())
+            patFilter->Request(Receiver->ChannelID().Sid());
          Start();
          return true;
          }
@@ -1806,17 +1831,18 @@ void cDevice::Detach(cReceiver *Receiver)
   bool receiversLeft = false;
   mutexReceiver.Lock();
   for (int i = 0; i < MAXRECEIVERS; i++) {
-      if (receiver[i] == Receiver) {
+      if (receiver[i] == Receiver)
          receiver[i] = NULL;
-         Receiver->device = NULL;
-         Receiver->Activate(false);
-         for (int n = 0; n < Receiver->numPids; n++)
-             DelPid(Receiver->pids[n]);
-         }
       else if (receiver[i])
          receiversLeft = true;
       }
+  if (patFilter && Receiver->ChannelID().Valid())
+     patFilter->Release(Receiver->ChannelID().Sid());
   mutexReceiver.Unlock();
+  Receiver->device = NULL;
+  Receiver->Activate(false);
+  for (int n = 0; n < Receiver->numPids; n++)
+      DelPid(Receiver->pids[n]);
   if (camSlot) {
      if (Receiver->priority > MINPRIORITY) { // priority check to avoid an infinite loop with the CAM slot's caPidReceiver
         camSlot->StartDecrypting();
