@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: dvbdevice.c 4.16.1.4 2019/05/28 15:55:44 kls Exp $
+ * $Id: dvbdevice.c 5.1 2021/06/09 09:12:25 kls Exp $
  */
 
 #include "dvbdevice.h"
@@ -34,6 +34,8 @@ static int DvbApiVersion = 0x0000; // the version of the DVB driver actually in
 
 #define SCR_RANDOM_TIMEOUT  500 // ms (add random value up to this when tuning SCR device to avoid lockups)
 
+#define TSBUFFERSIZE MEGABYTE(16)
+
 // --- DVB Parameter Maps ----------------------------------------------------
 
 const tDvbParameterMap PilotValues[] = {
@@ -365,6 +367,8 @@ cDvbFrontend::cDvbFrontend(int Adapter, int Frontend)
   frontend = Frontend;
   fd_frontend = -1;
   subsystemId = cDvbDeviceProbe::GetSubsystemId(adapter, frontend);
+  memset(&frontendInfo, 0, sizeof(frontendInfo));
+  strn0cpy(frontendInfo.name, "???", sizeof(frontendInfo.name));
   numModulations = 0;
   Open();
   QueryDeliverySystems();
@@ -554,7 +558,6 @@ private:
   cCondVar newSet;
   cDvbTuner *bondedTuner;
   bool bondedMaster;
-  bool SetFrontendType(const cChannel *Channel);
   cString GetBondingParams(const cChannel *Channel = NULL) const;
   cDvbTuner *GetBondedMaster(void);
   bool IsBondedMaster(void) const { return !bondedTuner || bondedMaster; }
@@ -790,7 +793,7 @@ bool cDvbTuner::IsTunedTo(const cChannel *Channel) const
 {
   if (tunerStatus == tsIdle)
      return false; // not tuned to
-  if (channel.Source() != Channel->Source() || channel.Transponder() != Channel->Transponder())
+  if (channel.Source() != Channel->Source() || channel.Transponder() != Channel->Transponder() || channel.Srate() != Channel->Srate())
      return false; // sufficient mismatch
   // Polarization is already checked as part of the Transponder.
   return strcmp(channel.Parameters(), Channel->Parameters()) == 0;
@@ -854,6 +857,7 @@ void cDvbTuner::ClearEventQueue(void) const
 bool cDvbTuner::GetFrontendStatus(fe_status_t &Status) const
 {
   ClearEventQueue();
+  Status = (fe_status_t)0; // initialize here to fix buggy drivers
   while (1) {
         if (ioctl(fd_frontend, FE_READ_STATUS, &Status) != -1)
            return true;
@@ -870,7 +874,7 @@ bool cDvbTuner::GetFrontendStatus(fe_status_t &Status) const
 bool cDvbTuner::GetSignalStats(int &Valid, double *Strength, double *Cnr, double *BerPre, double *BerPost, double *Per, int *Status) const
 {
   ClearEventQueue();
-  fe_status_t FeStatus;
+  fe_status_t FeStatus = (fe_status_t)0; // initialize here to fix buggy drivers
   dtv_property Props[MAXFRONTENDCMDS];
   dtv_properties CmdSeq;
   memset(&Props, 0, sizeof(Props));
@@ -898,7 +902,7 @@ bool cDvbTuner::GetSignalStats(int &Valid, double *Strength, double *Cnr, double
                   SETCMD(DTV_STAT_POST_TOTAL_BIT_COUNT, 0); }
   if (Per)      { SETCMD(DTV_STAT_ERROR_BLOCK_COUNT, 0);
                   SETCMD(DTV_STAT_TOTAL_BLOCK_COUNT, 0); }
-  if (ioctl(fd_frontend, FE_GET_PROPERTY, &CmdSeq) != 0) {
+  if (CmdSeq.num && ioctl(fd_frontend, FE_GET_PROPERTY, &CmdSeq) != 0) {
      esyslog("ERROR: frontend %d/%d: %m (%s:%d)", adapter, frontend, __FILE__, __LINE__);
      return false;
      }
@@ -1701,6 +1705,7 @@ void cDvbTuner::Action(void)
                      }
                   }
                tunerStatus = tsTuned;
+               device->SectionHandler()->SetStatus(true); // may have been turned off when retuning
                Timer.Set(tuneTimeout + (scr ? rand() % SCR_RANDOM_TIMEOUT : 0));
                if (positioner)
                   continue;
@@ -1733,6 +1738,14 @@ void cDvbTuner::Action(void)
                      isyslog("frontend %d/%d regained lock on channel %d (%s), tp %d", adapter, frontend, channel.Number(), channel.Name(), channel.Transponder());
                      LostLock = false;
                      }
+                  if (device->SdtFilter()->TransponderWrong()) {
+                     isyslog("frontend %d/%d is not receiving transponder %d for channel %d (%s) - retuning", adapter, frontend, channel.Transponder(), channel.Number(), channel.Name());
+                     device->SectionHandler()->SetStatus(false);
+                     tunerStatus = tsSet;
+                     lastDiseqc = NULL;
+                     lastSource = 0;
+                     continue;
+                     }
                   tunerStatus = tsLocked;
                   locked.Broadcast();
                   lastTimeoutReport = 0;
@@ -2126,11 +2139,17 @@ bool cDvbDevice::SetPid(cPidHandle *Handle, int Type, bool On)
   return true;
 }
 
+#define RB_NUM_SECTIONS 8 // default: 2 sections = 8192 bytes
+
 int cDvbDevice::OpenFilter(u_short Pid, u_char Tid, u_char Mask)
 {
   cString FileName = DvbName(DEV_DVB_DEMUX, adapter, frontend);
   int f = open(FileName, O_RDWR | O_NONBLOCK);
   if (f >= 0) {
+     if (Pid == EITPID) { // increase ringbuffer size for EIT
+        if (ioctl(f, DMX_SET_BUFFER_SIZE, MAX_SECTION_SIZE * RB_NUM_SECTIONS) < 0)
+           dsyslog("OpenFilter (pid=%d, tid=%02X): ioctl DMX_SET_BUFFER_SIZE failed", Pid, Tid);
+        }
      dmx_sct_filter_params sctFilterParams;
      memset(&sctFilterParams, 0, sizeof(sctFilterParams));
      sctFilterParams.pid = Pid;
@@ -2235,7 +2254,7 @@ bool cDvbDevice::ProvidesChannel(const cChannel *Channel, int Priority, bool *Ne
 
 bool cDvbDevice::ProvidesEIT(void) const
 {
-  return dvbTuner != NULL;
+  return dvbTuner != NULL && DeviceHooksProvidesEIT();
 }
 
 int cDvbDevice::NumProvidedSystems(void) const
@@ -2302,7 +2321,7 @@ bool cDvbDevice::OpenDvr(void)
   CloseDvr();
   fd_dvr = DvbOpen(DEV_DVB_DVR, adapter, frontend, O_RDONLY | O_NONBLOCK, true);
   if (fd_dvr >= 0)
-     tsBuffer = new cTSBuffer(fd_dvr, MEGABYTE(5), DeviceNumber() + 1);
+     tsBuffer = new cTSBuffer(fd_dvr, TSBUFFERSIZE, DeviceNumber() + 1);
   return fd_dvr >= 0;
 }
 
