@@ -8,9 +8,26 @@
  * Robert Schneider <Robert.Schneider@web.de> and Rolf Hakenes <hakenes@hippomi.de>.
  * Adapted to 'libsi' for VDR 1.3.0 by Marcel Wiesweg <marcel.wiesweg@gmx.de>.
  *
- * $Id: eit.c 4.5.1.2 2019/05/21 21:25:00 kls Exp $
+ * $Id: eit.c 5.4 2021/12/11 20:58:51 kls Exp $
  */
 
+// The various ways in which broadcasters handle (or screw up) their EPG:
+// - Some use the same version for all tables, and use unique event ids, which are the same in
+//   both the 0x5X and the 0x6X tables. And once an event has an id, it keeps it until it is
+//   no longer in the tables. Those are the good guys!
+// - Some use separate versions for each table (0x50, 0x51, ...).
+// - Some broadcast tables 0x5X and 0x6X, but use different event ids for the same event in both
+//   sets of tables, and sometimes even use different titles, short texts or descriptions.
+// - Some broadcast the full EPG only on one transponder (tables 0x6X), and on the actual transponder
+//   they provide only the present/following information.
+// - Some have overlapping events, especially when they mess up daylight saving time.
+// - Some use all new event ids every time they update their tables.
+// So, to bring order to chaos, VDR does as follows:
+// - Completely ignore table 0x4F.
+// - Once a schedule has seen events from 0x5X, tables 0x6X are ignored for that schedule.
+// - When looking up an event in its schedule, the start time is used for tables 0x6X, and the
+//   event id for tables 0x4E and 0x5X.
+
 #include "eit.h"
 #include <sys/time.h>
 #include "epg.h"
@@ -22,25 +39,57 @@
 
 #define DBGEIT 0
 
+// --- cEitTables ------------------------------------------------------------
+
+cEitTables::cEitTables(void)
+{
+  complete = false;
+  tableStart = 0;
+  tableEnd = 0;
+}
+
+bool cEitTables::Check(uchar TableId, uchar Version, int SectionNumber)
+{
+  int ti = Index(TableId);
+  return sectionSyncer[ti].Check(Version, SectionNumber);
+}
+
+bool cEitTables::Processed(uchar TableId, uchar LastTableId, int SectionNumber, int LastSectionNumber, int SegmentLastSectionNumber)
+{
+  bool Result = false;
+  int ti = Index(TableId);
+  int LastIndex = Index(LastTableId);
+  complete = false;
+  if (sectionSyncer[ti].Processed(SectionNumber, LastSectionNumber, SegmentLastSectionNumber)) {
+     Result = true; // the table with TableId is complete
+     for (int i = 0; i <= LastIndex; i++) {
+         if (!sectionSyncer[i].Complete())
+            return Result;
+         }
+     complete = true; // all tables have been processed
+     }
+  return Result;
+}
+
 // --- cEIT ------------------------------------------------------------------
 
 class cEIT : public SI::EIT {
 public:
-  cEIT(cSectionSyncerHash &SectionSyncerHash, int Source, u_char Tid, const u_char *Data);
+  cEIT(cEitTablesHash &EitTablesHash, int Source, u_char Tid, const u_char *Data);
   };
 
-cEIT::cEIT(cSectionSyncerHash &SectionSyncerHash, int Source, u_char Tid, const u_char *Data)
+cEIT::cEIT(cEitTablesHash &EitTablesHash, int Source, u_char Tid, const u_char *Data)
 :SI::EIT(Data, false)
 {
   if (!CheckCRCAndParse())
      return;
-  int HashId = Tid * getServiceId();
-  cSectionSyncerEntry *SectionSyncerEntry = SectionSyncerHash.Get(HashId);
-  if (!SectionSyncerEntry) {
-     SectionSyncerEntry = new cSectionSyncerEntry;
-     SectionSyncerHash.Add(SectionSyncerEntry, HashId);
+  int HashId = getServiceId();
+  cEitTables *EitTables = EitTablesHash.Get(HashId);
+  if (!EitTables) {
+     EitTables = new cEitTables;
+     EitTablesHash.Add(EitTables, HashId);
      }
-  bool Process = SectionSyncerEntry->Sync(getVersionNumber(), getSectionNumber(), getLastSectionNumber());
+  bool Process = EitTables->Check(Tid, getVersionNumber(), getSectionNumber());
   if (Tid != 0x4E && !Process) // we need to set the 'seen' tag to watch the running status of the present/following event
      return;
 
@@ -50,10 +99,8 @@ cEIT::cEIT(cSectionSyncerHash &SectionSyncerHash, int Source, u_char Tid, const
 
   cStateKey ChannelsStateKey;
   cChannels *Channels = cChannels::GetChannelsWrite(ChannelsStateKey, 10);
-  if (!Channels) {
-     SectionSyncerEntry->Repeat(); // let's not miss any section of the EIT
+  if (!Channels)
      return;
-     }
   tChannelID channelID(Source, getOriginalNetworkId(), getTransportStreamId(), getServiceId());
   cChannel *Channel = Channels->GetByChannelID(channelID, true);
   if (!Channel || EpgHandlers.IgnoreChannel(Channel)) {
@@ -64,7 +111,6 @@ cEIT::cEIT(cSectionSyncerHash &SectionSyncerHash, int Source, u_char Tid, const
   cStateKey SchedulesStateKey;
   cSchedules *Schedules = cSchedules::GetSchedulesWrite(SchedulesStateKey, 10);
   if (!Schedules) {
-     SectionSyncerEntry->Repeat(); // let's not miss any section of the EIT
      ChannelsStateKey.Remove(false);
      return;
      }
@@ -79,10 +125,16 @@ cEIT::cEIT(cSectionSyncerHash &SectionSyncerHash, int Source, u_char Tid, const
   bool handledExternally = EpgHandlers.HandledExternally(Channel);
   cSchedule *pSchedule = (cSchedule *)Schedules->GetSchedule(Channel, true);
 
+  if (pSchedule->OnActualTp(Tid) && (Tid & 0xF0) == 0x60) {
+     SchedulesStateKey.Remove(false);
+     ChannelsStateKey.Remove(false);
+     return;
+     }
+
   bool Empty = true;
   bool Modified = false;
-  time_t LingerLimit = Now - Setup.EPGLinger * 60;
-  time_t SegmentStart = 0;
+  time_t LingerLimit = Now - EPG_LINGER_TIME;
+  time_t SegmentStart = 0; // these are actually "section" start/end times
   time_t SegmentEnd = 0;
   struct tm t = { 0 };
   localtime_r(&Now, &t); // this initializes the time zone in 't'
@@ -103,9 +155,19 @@ cEIT::cEIT(cSectionSyncerHash &SectionSyncerHash, int Source, u_char Tid, const
       if (!SegmentStart)
          SegmentStart = StartTime;
       SegmentEnd = StartTime + Duration;
+      if (Tid == 0x4E) {
+         if (getSectionNumber() == 0)
+            EitTables->SetTableStart(SegmentStart);
+         else
+            EitTables->SetTableEnd(SegmentEnd);
+         }
       cEvent *newEvent = NULL;
       cEvent *rEvent = NULL;
-      cEvent *pEvent = (cEvent *)pSchedule->GetEvent(SiEitEvent.getEventId(), StartTime);
+      cEvent *pEvent = NULL;
+      if (Tid == 0x4E || (Tid & 0xF0) == 0x50)
+         pEvent = const_cast<cEvent *>(pSchedule->GetEventById(SiEitEvent.getEventId()));
+      else
+         pEvent = const_cast<cEvent *>(pSchedule->GetEventByTime(StartTime));
       if (!pEvent || handledExternally) {
          if (handledExternally && !EpgHandlers.IsUpdate(SiEitEvent.getEventId(), StartTime, Tid, getVersionNumber()))
             continue;
@@ -121,10 +183,13 @@ cEIT::cEIT(cSectionSyncerHash &SectionSyncerHash, int Source, u_char Tid, const
          // We have found an existing event, either through its event ID or its start time.
          pEvent->SetSeen();
          uchar TableID = max(pEvent->TableID(), uchar(0x4E)); // for backwards compatibility, table ids less than 0x4E are treated as if they were "present"
-         // If the new event has a higher table ID, let's skip it.
-         // The lower the table ID, the more "current" the information.
-         if (Tid > TableID)
+         // We never overwrite present/following with events from other tables:
+         if (TableID == 0x4E && Tid != 0x4E)
             continue;
+         if (pEvent->HasTimer()) {
+            if (pEvent->StartTime() != StartTime || pEvent->Duration() != Duration)
+               dsyslog("channel %d (%s) event %s times changed to %s-%s", Channel->Number(), Channel->Name(), *pEvent->ToDescr(), *TimeString(StartTime), *TimeString(StartTime + Duration));
+            }
          EpgHandlers.SetEventID(pEvent, SiEitEvent.getEventId()); // unfortunately some stations use different event ids for the same event in different tables :-(
          EpgHandlers.SetStartTime(pEvent, StartTime);
          EpgHandlers.SetDuration(pEvent, Duration);
@@ -256,7 +321,7 @@ cEIT::cEIT(cSectionSyncerHash &SectionSyncerHash, int Source, u_char Tid, const
                  cSchedule *rSchedule = (cSchedule *)Schedules->GetSchedule(tChannelID(Source, Channel->Nid(), Channel->Tid(), tsed->getReferenceServiceId()));
                  if (!rSchedule)
                     break;
-                 rEvent = (cEvent *)rSchedule->GetEvent(tsed->getReferenceEventId());
+                 rEvent = (cEvent *)rSchedule->GetEventById(tsed->getReferenceEventId());
                  if (!rEvent)
                     break;
                  EpgHandlers.SetTitle(pEvent, rEvent->Title());
@@ -302,11 +367,15 @@ cEIT::cEIT(cSectionSyncerHash &SectionSyncerHash, int Source, u_char Tid, const
             case SI::ComponentDescriptorTag: {
                  SI::ComponentDescriptor *cd = (SI::ComponentDescriptor *)d;
                  uchar Stream = cd->getStreamContent();
+                 uchar Ext = cd->getStreamContentExt();
                  uchar Type = cd->getComponentType();
-                 if (1 <= Stream && Stream <= 6 && Type != 0) { // 1=MPEG2-video, 2=MPEG1-audio, 3=subtitles, 4=AC3-audio, 5=H.264-video, 6=HEAAC-audio
+                 if ((1 <= Stream && Stream <= 6 && Type != 0) // 1=MPEG2-video, 2=MPEG1-audio, 3=subtitles, 4=AC3-audio, 5=H.264-video, 6=HEAAC-audio
+                    || (Stream == 9 && Ext < 2)) {             // 0x09=HEVC-video, 0x19=AC-4-audio
                     if (!Components)
                        Components = new cComponents;
                     char buffer[Utf8BufSize(256)];
+                    if (Stream == 9)
+                       Stream |= Ext << 4;
                     Components->SetComponent(Components->NumComponents(), Stream, Type, I18nNormalizeLanguageCode(cd->languageCode), cd->description.getText(buffer, sizeof(buffer)));
                     }
                  }
@@ -352,14 +421,20 @@ cEIT::cEIT(cSectionSyncerHash &SectionSyncerHash, int Source, u_char Tid, const
         pSchedule->ClrRunningStatus(Channel);
      pSchedule->SetPresentSeen();
      }
-  if (Modified) {
-     EpgHandlers.SortSchedule(pSchedule);
-     EpgHandlers.DropOutdated(pSchedule, SegmentStart, SegmentEnd, Tid, getVersionNumber());
-     pSchedule->SetModified();
+  if (Process) {
+     bool Complete = EitTables->Processed(Tid, getLastTableId(), getSectionNumber(), getLastSectionNumber(), getSegmentLastSectionNumber());
+     if (Modified && (Tid >= 0x50 || Complete)) { // we process the 0x5X tables segment by segment, but 0x4E only if we have received ALL its segments (0 and 1, i.e. "present" and "following")
+        if (Tid == 0x4E && getLastSectionNumber() == 1) {
+           SegmentStart = EitTables->TableStart();
+           SegmentEnd = EitTables->TableEnd();
+           }
+        EpgHandlers.SortSchedule(pSchedule);
+        EpgHandlers.DropOutdated(pSchedule, SegmentStart, SegmentEnd, Tid, getVersionNumber());
+        }
      }
+  EpgHandlers.EndSegmentTransfer(Modified);
   SchedulesStateKey.Remove(Modified);
   ChannelsStateKey.Remove(ChannelsModified);
-  EpgHandlers.EndSegmentTransfer(Modified);
 }
 
 // --- cTDT ------------------------------------------------------------------
@@ -391,7 +466,9 @@ cTDT::cTDT(const u_char *Data)
   if (abs(diff) > MAX_TIME_DIFF) {
      mutex.Lock();
      if (abs(diff) > MAX_ADJ_DIFF) {
-        if (stime(&dvbtim) == 0)
+        timespec ts = {};
+        ts.tv_sec = dvbtim;
+        if (clock_settime(CLOCK_REALTIME, &ts) == 0)
            isyslog("system time changed from %s (%ld) to %s (%ld)", *TimeToString(loctim), loctim, *TimeToString(dvbtim), dvbtim);
         else
            esyslog("ERROR while setting system time: %m");
@@ -416,7 +493,7 @@ time_t cEitFilter::disableUntil = 0;
 
 cEitFilter::cEitFilter(void)
 {
-  Set(0x12, 0x40, 0xC0);  // event info now&next actual/other TS (0x4E/0x4F), future actual/other TS (0x5X/0x6X)
+  Set(0x12, 0x40, 0xC0);  // event info present&following actual/other TS (0x4E/0x4F), future actual/other TS (0x5X/0x6X)
   Set(0x14, 0x70);        // TDT
 }
 
@@ -424,7 +501,7 @@ void cEitFilter::SetStatus(bool On)
 {
   cMutexLock MutexLock(&mutex);
   cFilter::SetStatus(On);
-  sectionSyncerHash.Clear();
+  eitTablesHash.Clear();
 }
 
 void cEitFilter::SetDisableUntil(time_t Time)
@@ -443,8 +520,8 @@ void cEitFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
      }
   switch (Pid) {
     case 0x12: {
-         if (Tid >= 0x4E && Tid <= 0x6F)
-            cEIT EIT(sectionSyncerHash, Source(), Tid, Data);
+         if (Tid == 0x4E || Tid >= 0x50 && Tid <= 0x6F) // we ignore 0x4F, which only causes trouble
+            cEIT EIT(eitTablesHash, Source(), Tid, Data);
          }
          break;
     case 0x14: {
