@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: remux.c 4.7.1.1 2019/03/15 13:13:40 kls Exp $
+ * $Id: remux.c 5.1 2021/05/11 20:47:31 kls Exp $
  */
 
 #include "remux.h"
@@ -962,9 +962,9 @@ uchar *cEitGenerator::AddParentalRatingDescriptor(uchar *p, uchar ParentalRating
 {
   *p++ = SI::ParentalRatingDescriptorTag;
   *p++ = 0x04; // descriptor length
-  *p++ = 'D';  // country code
-  *p++ = 'E';
-  *p++ = 'U';
+  *p++ = '9';  // country code "902" ("All countries") -> EN 300 468 / 6.2.28; www.dvbservices.com/country_codes/index.php
+  *p++ = '0';
+  *p++ = '2';
   *p++ = ParentalRating;
   return p;
 }
@@ -1292,7 +1292,7 @@ int cMpeg2Parser::Parse(const uchar *Data, int Length, int Pid)
          tsPayload.Statistics();
          break;
          }
-      if (tsPayload.AtPayloadStart() // stop at any new payload start to have the buffer refilled if necessary
+      if (tsPayload.AtPayloadStart() // stop at a new payload start to have the buffer refilled if necessary
          || tsPayload.Eof()) // or if we're out of data
          break;
       }
@@ -1445,7 +1445,7 @@ int cH264Parser::Parse(const uchar *Data, int Length, int Pid)
            default: ;
            }
          }
-      if (tsPayload.AtPayloadStart() // stop at any new payload start to have the buffer refilled if necessary
+      if (tsPayload.AtPayloadStart() // stop at a new payload start to have the buffer refilled if necessary
          || tsPayload.Eof()) // or if we're out of data
          break;
       }
@@ -1603,7 +1603,7 @@ int cH265Parser::Parse(const uchar *Data, int Length, int Pid)
             break;
             }
          }
-      if (tsPayload.AtPayloadStart() // stop at any new payload start to have the buffer refilled if necessary
+      if (tsPayload.AtPayloadStart() // stop at a new payload start to have the buffer refilled if necessary
          || tsPayload.Eof()) // or if we're out of data
          break;
       }
