@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: skinlcars.c 4.6 2017/11/08 10:10:30 kls Exp $
+ * $Id: skinlcars.c 5.3 2021/07/01 15:40:46 kls Exp $
  */
 
 // "Star Trek: The Next Generation"(R) is a registered trademark of Paramount Pictures,
@@ -1213,14 +1213,15 @@ void cSkinLCARSDisplayMenu::DrawTimer(const cTimer *Timer, int y, bool MultiRec)
   osd->DrawRectangle(xs00, y, xs03 - 1, y + lineHeight - 1, ColorBg);
   cString Date;
   if (Timer->Recording())
-     Date = cString::sprintf("-%s", *TimeString(Timer->StopTime()));
+     Date = cString::sprintf("-%s", *TimeString(Timer->StopTimeEvent()));
   else {
      time_t Now = time(NULL);
+     time_t StartTime = Timer->StartTimeEvent();
      cString Today = WeekDayName(Now);
-     cString Time = TimeString(Timer->StartTime());
-     cString Day = WeekDayName(Timer->StartTime());
-     if (Timer->StartTime() > Now + 6 * SECSINDAY)
-        Date = DayDateTime(Timer->StartTime());
+     cString Time = TimeString(StartTime);
+     cString Day = WeekDayName(StartTime);
+     if (StartTime > Now + 6 * SECSINDAY)
+        Date = DayDateTime(StartTime);
      else if (strcmp(Day, Today) != 0)
         Date = cString::sprintf("%s %s", *Day, *Time);
      else
@@ -1264,7 +1265,9 @@ void cSkinLCARSDisplayMenu::DrawTimers(void)
                if (y + lineHeight > ys05)
                   break;
                if (const cTimer *Timer = SortedTimers[i]) {
-                  if (Timer->Recording()) {
+                  if (Timer->IsPatternTimer())
+                     SortedTimers[i] = NULL;
+                  else if (Timer->Recording()) {
                      if (Timer->Remote()) {
                         if (!Device && Timer->HasFlags(tfActive)) {
                            DrawTimer(Timer, y, false);
@@ -1690,11 +1693,23 @@ void cSkinLCARSDisplayMenu::SetRecording(const cRecording *Recording)
   cString t = cString::sprintf("%s  %s  %s", *DateString(Recording->Start()), *TimeString(Recording->Start()), Info->ChannelName() ? Info->ChannelName() : "");
   ts.Set(osd, xl, y, xi01 - xl, yi01 - y, t, font, Theme.Color(clrEventTime), Theme.Color(clrBackground));
   y += ts.Height();
+  int xt = xi01;
   if (Info->GetEvent()->ParentalRating()) {
      cString buffer = cString::sprintf(" %s ", *Info->GetEvent()->GetParentalRatingString());
      const cFont *font = cFont::GetFont(fontSml);
      int w = font->Width(buffer);
-     osd->DrawText(xi01 - w, y, buffer, Theme.Color(clrMenuFrameFg), frameColor, font, w);
+     osd->DrawText(xt - w, y, buffer, Theme.Color(clrMenuFrameFg), frameColor, font, w);
+     xt -= w + xi02 - xi01;
+     }
+  if (Info->Errors() > 0) {
+     cString buffer = cString::sprintf(" %d %s ", Info->Errors(), tr("errors"));
+     const cFont *font = cFont::GetFont(fontSml);
+     int w = font->Width(buffer);
+     osd->DrawText(xt - w, y, buffer, Theme.Color(clrMenuFrameFg), frameColor, font, w);
+     xt -= w + xi02 - xi01;
+     }
+  if (xt != xi01) {
+     const cFont *font = cFont::GetFont(fontSml);
      int yb = y + font->Height();
      osd->DrawRectangle(xi02, y, xi02 + lineHeight / 2 - 1, yb - 1, frameColor);
      osd->DrawEllipse  (xi02 + lineHeight / 2, y, xi03 - 1, yb - 1, frameColor, 5);
@@ -1741,12 +1756,13 @@ void cSkinLCARSDisplayMenu::Flush(void)
 {
   if (MenuCategory() == mcMain) {
      cDevice *Device = cDevice::PrimaryDevice();
+     cMutexLock ControlMutexLock;
      if (!Device->Replaying() || Device->Transferring()) {
         LOCK_CHANNELS_READ;
         const cChannel *Channel = Channels->GetByNumber(cDevice::PrimaryDevice()->CurrentChannel());
         DrawLive(Channel);
         }
-     else if (cControl *Control = cControl::Control(true))
+     else if (cControl *Control = cControl::Control(ControlMutexLock, true))
         DrawPlay(Control);
      DrawTimers();
      DrawDevices();
