@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: timers.c 4.18.1.1 2019/05/23 10:02:45 kls Exp $
+ * $Id: timers.c 5.17 2021/05/03 14:06:12 kls Exp $
  */
 
 #include "timers.h"
@@ -27,10 +27,11 @@ cTimer::cTimer(bool Instant, bool Pause, const cChannel *Channel)
 {
   id = 0;
   startTime = stopTime = 0;
-  scheduleState = -1;
+  scheduleStateSet = scheduleStateSpawn = scheduleStateAdjust = -1;
   deferred = 0;
   pending = inVpsMargin = false;
   flags = tfNone;
+  *pattern = 0;
   *file = 0;
   aux = NULL;
   remote = NULL;
@@ -57,8 +58,11 @@ cTimer::cTimer(bool Instant, bool Pause, const cChannel *Channel)
               tstart = Event->Vps();
               }
            else {
-              tstop  += Setup.MarginStop * 60;
-              tstart -= Setup.MarginStart * 60;
+              int MarginStart = 0;
+              int MarginStop  = 0;
+              CalcMargins(MarginStart, MarginStop, Event);
+              tstart -= MarginStart;
+              tstop  += MarginStop;
               }
            day = SetTime(tstart, 0);
            struct tm *time = localtime_r(&tstart, &tm_r);
@@ -81,27 +85,121 @@ cTimer::cTimer(bool Instant, bool Pause, const cChannel *Channel)
      snprintf(file, sizeof(file), "%s%s", Setup.MarkInstantRecord ? "@" : "", *Setup.NameInstantRecord ? Setup.NameInstantRecord : channel->Name());
 }
 
-cTimer::cTimer(const cEvent *Event)
+static bool MatchPattern(const char *Pattern, const char *Title, cString *Before = NULL, cString *Match = NULL, cString *After = NULL)
+{
+  if (Title) {
+     bool AvoidDuplicates = startswith(Pattern, TIMERPATTERN_AVOID);
+     if (AvoidDuplicates)
+        Pattern++;
+     if (strcmp(Pattern, "*") == 0) {
+        if (Before)
+           *Before = "";
+        if (Match)
+           *Match = Title;
+        if (After)
+           *After = "";
+        return true;
+        }
+     bool AnchorBegin = startswith(Pattern, TIMERPATTERN_BEGIN);
+     if (AnchorBegin)
+        Pattern++;
+     bool AnchorEnd = endswith(Pattern, TIMERPATTERN_END);
+     cNullTerminate nt;
+     if (AnchorEnd)
+        nt.Set(const_cast<char *>(Pattern + strlen(Pattern) - 1));
+     if (AnchorBegin && AnchorEnd) {
+        if (strcmp(Title, Pattern) == 0) {
+           if (Before)
+              *Before = "";
+           if (Match)
+              *Match = Title;
+           if (After)
+              *After = "";
+           return true;
+           }
+        }
+     else if (AnchorBegin) {
+        if (strstr(Title, Pattern) == Title) {
+           if (Before)
+              *Before = "";
+           if (Match)
+              *Match = Pattern;
+           if (After)
+              *After = cString(Title + strlen(Pattern));
+           return true;
+           }
+        }
+     else if (AnchorEnd) {
+        if (endswith(Title, Pattern)) {
+           if (Before)
+              *Before = cString(Title, Title + strlen(Title) - strlen(Pattern));
+           if (Match)
+              *Match = Pattern;
+           if (After)
+              *After = "";
+           return true;
+           }
+        }
+     else if (const char *p = strstr(Title, Pattern)) {
+        if (Before)
+           *Before = cString(Title, p);
+        if (Match)
+           *Match = Pattern;
+        if (After)
+           *After = cString(p + strlen(Pattern));
+        return true;
+        }
+     }
+  return false;
+}
+
+static cString MakePatternFileName(const char *Pattern, const char *Title, const char *Episode, const char *File)
+{
+  if (!Pattern || !Title || !File)
+     return NULL;
+  cString Before = "";
+  cString Match = "";
+  cString After = "";
+  if (MatchPattern(Pattern, Title, &Before, &Match, &After)) {
+     char *Result = strdup(File);
+     Result = strreplace(Result, TIMERMACRO_TITLE, Title);
+     if (!isempty(Episode)) // the event might not yet have a "short text", so we leave this to the actual recording
+        Result = strreplace(Result, TIMERMACRO_EPISODE, Episode);
+     Result = strreplace(Result, TIMERMACRO_BEFORE, Before);
+     Result = strreplace(Result, TIMERMACRO_MATCH, Match);
+     Result = strreplace(Result, TIMERMACRO_AFTER, After);
+     return cString(Result, true);
+     }
+  return NULL;
+}
+
+cTimer::cTimer(const cEvent *Event, const char *FileName, const cTimer *PatternTimer)
 {
   id = 0;
   startTime = stopTime = 0;
-  scheduleState = -1;
+  scheduleStateSet = scheduleStateSpawn = scheduleStateAdjust = -1;
   deferred = 0;
   pending = inVpsMargin = false;
   flags = tfActive;
+  *pattern = 0;
   *file = 0;
   aux = NULL;
   remote = NULL;
   event = NULL;
-  if (Event->Vps() && Setup.UseVps)
-     SetFlags(tfVps);
+  if (!PatternTimer || PatternTimer->HasFlags(tfVps)) {
+     if (Event->Vps() && (PatternTimer || Setup.UseVps))
+        SetFlags(tfVps);
+     }
   LOCK_CHANNELS_READ;
   channel = Channels->GetByChannelID(Event->ChannelID(), true);
   time_t tstart = (flags & tfVps) ? Event->Vps() : Event->StartTime();
   time_t tstop = tstart + Event->Duration();
   if (!(HasFlags(tfVps))) {
-     tstop  += Setup.MarginStop * 60;
-     tstart -= Setup.MarginStart * 60;
+     int MarginStart = 0;
+     int MarginStop  = 0;
+     CalcMargins(MarginStart, MarginStop, Event);
+     tstart -= MarginStart;
+     tstop  += MarginStop;
      }
   struct tm tm_r;
   struct tm *time = localtime_r(&tstart, &tm_r);
@@ -112,11 +210,12 @@ cTimer::cTimer(const cEvent *Event)
   stop = time->tm_hour * 100 + time->tm_min;
   if (stop >= 2400)
      stop -= 2400;
-  priority = Setup.DefaultPriority;
-  lifetime = Setup.DefaultLifetime;
-  const char *Title = Event->Title();
-  if (!isempty(Title))
-     Utf8Strn0Cpy(file, Event->Title(), sizeof(file));
+  priority = PatternTimer ? PatternTimer->Priority() : Setup.DefaultPriority;
+  lifetime = PatternTimer ? PatternTimer->Lifetime() : Setup.DefaultLifetime;
+  if (!FileName)
+     FileName = Event->Title();
+  if (!isempty(FileName))
+     Utf8Strn0Cpy(file, FileName, sizeof(file));
   SetEvent(Event);
 }
 
@@ -144,7 +243,7 @@ cTimer& cTimer::operator= (const cTimer &Timer)
      id           = Timer.id;
      startTime    = Timer.startTime;
      stopTime     = Timer.stopTime;
-     scheduleState = -1;
+     scheduleStateSet = scheduleStateSpawn = scheduleStateAdjust = -1;
      deferred     = 0;
      pending      = Timer.pending;
      inVpsMargin  = Timer.inVpsMargin;
@@ -156,6 +255,7 @@ cTimer& cTimer::operator= (const cTimer &Timer)
      stop         = Timer.stop;
      priority     = Timer.priority;
      lifetime     = Timer.lifetime;
+     strncpy(pattern, Timer.pattern, sizeof(pattern));
      strncpy(file, Timer.file, sizeof(file));
      free(aux);
      aux = Timer.aux ? strdup(Timer.aux) : NULL;
@@ -170,6 +270,18 @@ cTimer& cTimer::operator= (const cTimer &Timer)
   return *this;
 }
 
+void cTimer::CalcMargins(int &MarginStart, int &MarginStop, const cEvent *Event)
+{
+  MarginStart = Setup.MarginStart * 60;
+  MarginStop  = Setup.MarginStop * 60;
+  // To make sure the timer gets assigned to the correct event, we must
+  // make sure that this is the only event that overlaps 100%:
+  if (const cEvent *e = dynamic_cast<const cEvent *>(Event->Prev()))
+     MarginStart = max(0, min(MarginStart, e->Duration() - 60));
+  if (const cEvent *e = dynamic_cast<const cEvent *>(Event->Next()))
+     MarginStop = max(0, min(MarginStop, e->Duration() - 60));
+}
+
 int cTimer::Compare(const cListObject &ListObject) const
 {
   const cTimer *ti = (const cTimer *)&ListObject;
@@ -178,20 +290,37 @@ int cTimer::Compare(const cListObject &ListObject) const
   int r = t1 - t2;
   if (r == 0)
      r = ti->priority - priority;
+  if (IsPatternTimer() ^ ti->IsPatternTimer()) {
+     if (IsPatternTimer())
+        r = 1;
+     else
+        r = -1;
+     }
+  else if (IsPatternTimer() && ti->IsPatternTimer())
+     r = strcoll(Pattern(), ti->Pattern());
   return r;
 }
 
+cString cTimer::PatternAndFile(void) const
+{
+  if (IsPatternTimer())
+     return cString::sprintf("{%s}%s", pattern, file);
+  return file;
+}
+
 cString cTimer::ToText(bool UseChannelID) const
 {
+  strreplace(pattern, ':', '|');
   strreplace(file, ':', '|');
-  cString buffer = cString::sprintf("%u:%s:%s:%04d:%04d:%d:%d:%s:%s", flags, UseChannelID ? *Channel()->GetChannelID().ToString() : *itoa(Channel()->Number()), *PrintDay(day, weekdays, true), start, stop, priority, lifetime, file, aux ? aux : "");
+  cString buffer = cString::sprintf("%u:%s:%s:%04d:%04d:%d:%d:%s:%s", flags, UseChannelID ? *Channel()->GetChannelID().ToString() : *itoa(Channel()->Number()), *PrintDay(day, weekdays, true), start, stop, priority, lifetime, *PatternAndFile(), aux ? aux : "");
+  strreplace(pattern, '|', ':');
   strreplace(file, '|', ':');
   return buffer;
 }
 
 cString cTimer::ToDescr(void) const
 {
-  return cString::sprintf("%d%s%s (%d %04d-%04d %s'%s')", Id(), remote ? "@" : "", remote ? remote : "", Channel()->Number(), start, stop, HasFlags(tfVps) ? "VPS " : "", file);
+  return cString::sprintf("%d%s%s (%d %04d-%04d %s'%s')", Id(), remote ? "@" : "", remote ? remote : "", Channel()->Number(), start, stop, HasFlags(tfVps) ? "VPS " : "", *PatternAndFile());
 }
 
 int cTimer::TimeToInt(int t)
@@ -313,7 +442,7 @@ bool cTimer::Parse(const char *s)
   //XXX results in an empty string (this first occurred when the EIT gathering
   //XXX was put into a separate thread - don't know why this happens...
   //XXX As a cure we copy the original string and add a blank.
-  //XXX If anybody can shed some light on why sscanf() failes here, I'd love
+  //XXX If anybody can shed some light on why sscanf() fails here, I'd love
   //XXX to hear about that!
   char *s2 = NULL;
   int l2 = strlen(s);
@@ -332,7 +461,18 @@ bool cTimer::Parse(const char *s)
         }
      //TODO add more plausibility checks
      result = ParseDay(daybuffer, day, weekdays);
-     Utf8Strn0Cpy(file, filebuffer, sizeof(file));
+     char *fb = filebuffer;
+     if (*fb == '{') {
+        if (char *p = strchr(fb, '}')) {
+           *p = 0;
+           Utf8Strn0Cpy(pattern, fb + 1, sizeof(pattern));
+           strreplace(pattern, '|', ':');
+           fb = p + 1;
+           }
+        }
+     else
+        *pattern = 0;
+     Utf8Strn0Cpy(file, fb, sizeof(file));
      strreplace(file, '|', ':');
      LOCK_CHANNELS_READ;
      if (isnumber(channelbuffer))
@@ -404,6 +544,11 @@ time_t cTimer::SetTime(time_t t, int SecondsFromMidnight)
   return mktime(&tm);
 }
 
+void cTimer::SetPattern(const char *Pattern)
+{
+  Utf8Strn0Cpy(pattern, Pattern, sizeof(pattern));
+}
+
 void cTimer::SetFile(const char *File)
 {
   if (!isempty(File))
@@ -451,19 +596,35 @@ bool cTimer::Matches(time_t t, bool Directly, int Margin) const
         day = 0;
      }
 
+  if (IsPatternTimer())
+     return false; // we only need to have start/stopTime initialized
+
   if (t < deferred)
      return false;
   deferred = 0;
 
   if (HasFlags(tfActive)) {
-     if (HasFlags(tfVps) && event && event->Vps()) {
-        if (Margin || !Directly) {
-           startTime = event->StartTime();
-           stopTime = event->EndTime();
-           if (!Margin) { // this is an actual check
-              if (event->Schedule()->PresentSeenWithin(EITPRESENTFOLLOWINGRATE)) // VPS control can only work with up-to-date events...
-                 return event->IsRunning(true);
-              return startTime <= t && t < stopTime; // ...otherwise we fall back to normal timer handling
+     if (event) {
+        if (HasFlags(tfVps)) {
+           if (event->Vps()) {
+              if (Margin || !Directly) {
+                 startTime = event->StartTime();
+                 stopTime = event->EndTime();
+                 if (!Margin) { // this is an actual check
+                    if (event->Schedule()->PresentSeenWithin(EITPRESENTFOLLOWINGRATE)) // VPS control can only work with up-to-date events...
+                       return event->IsRunning(true);
+                    // ...otherwise we fall back to normal timer handling below (note: Margin == 0!)
+                    }
+                 }
+              }
+           }
+        else if (HasFlags(tfSpawned)) {
+           if (!Margin && !Directly) { // this is an actual check
+              // The spawned timer's start-/stopTimes are adjusted to the event's times in AdjustSpawnedTimer().
+              // However, in order to make sure the timer is set to the correct event, the margins at begin
+              // end end are limited by the durations of the events before and after this timer's event.
+              // The recording, though, shall always use the full start/stop margins, hence this calculation:
+              return event->StartTime() - Setup.MarginStart * 60 <= t && t < event->EndTime() + Setup.MarginStop * 60;
               }
            }
         }
@@ -483,6 +644,21 @@ eTimerMatch cTimer::Matches(const cEvent *Event, int *Overlap) const
   // gets 200 added to the FULLMATCH.
   if (channel->GetChannelID() == Event->ChannelID()) {
      bool UseVps = HasFlags(tfVps) && Event->Vps();
+     if (IsPatternTimer()) {
+        if (startswith(Pattern(), TIMERPATTERN_AVOID)) {
+           cString FileName = MakePatternFileName(Pattern(), Event->Title(), Event->ShortText(), File());
+           if (*FileName) {
+              const char *p = strgetlast(*FileName, FOLDERDELIMCHAR);
+              if (DoneRecordingsPattern.Contains(p))
+                 return tmNone;
+              }
+           else
+              return tmNone;
+           }
+        else if (!MatchPattern(Pattern(), Event->Title()))
+           return tmNone;
+        UseVps = false;
+        }
      Matches(UseVps ? Event->Vps() : Event->StartTime(), true);
      int overlap = 0;
      if (UseVps) {
@@ -499,8 +675,11 @@ eTimerMatch cTimer::Matches(const cEvent *Event, int *Overlap) const
            overlap = FULLMATCH;
         else if (stopTime <= Event->StartTime() || Event->EndTime() <= startTime)
            overlap = 0;
-        else
+        else {
            overlap = (min(stopTime, Event->EndTime()) - max(startTime, Event->StartTime())) * FULLMATCH / max(Event->Duration(), 1);
+           if (IsPatternTimer() && overlap > 0)
+              overlap = FULLMATCH;
+           }
         }
      startTime = stopTime = 0;
      if (Overlap)
@@ -514,10 +693,13 @@ eTimerMatch cTimer::Matches(const cEvent *Event, int *Overlap) const
 
 bool cTimer::Expired(void) const
 {
-  return IsSingleEvent()
-      && !Recording()
-      && StopTime() + EXPIRELATENCY <= time(NULL)
-      && (!HasFlags(tfVps) || !event || !event->Vps() || event->EndTime() + EXPIRELATENCY <= time(NULL));
+  if (IsSingleEvent() && !Recording()) {
+     time_t ExpireTime = StopTimeEvent();
+     if (HasFlags(tfVps))
+        ExpireTime += EXPIRELATENCY;
+     return ExpireTime <= time(NULL);
+     }
+  return false;
 }
 
 time_t cTimer::StartTime(void) const
@@ -534,6 +716,28 @@ time_t cTimer::StopTime(void) const
   return stopTime;
 }
 
+time_t cTimer::StartTimeEvent(void) const
+{
+  if (event) {
+     if (HasFlags(tfVps) && event->Vps())
+        return event->StartTime();
+     else if (HasFlags(tfSpawned))
+        return event->StartTime() - Setup.MarginStart * 60;
+     }
+  return StartTime();
+}
+
+time_t cTimer::StopTimeEvent(void) const
+{
+  if (event) {
+     if (HasFlags(tfVps) && event->Vps())
+        return event->EndTime();
+     else if (HasFlags(tfSpawned))
+        return event->EndTime() + Setup.MarginStop * 60;
+     }
+  return StopTime();
+}
+
 #define EPGLIMITBEFORE   (1 * 3600) // Time in seconds before a timer's start time and
 #define EPGLIMITAFTER    (1 * 3600) // after its stop time within which EPG events will be taken into consideration.
 
@@ -542,11 +746,121 @@ void cTimer::SetId(int Id)
   id = Id;
 }
 
-bool cTimer::SetEventFromSchedule(const cSchedules *Schedules)
+cTimer *cTimer::SpawnPatternTimer(const cEvent *Event, cTimers *Timers)
 {
+  cString FileName = MakePatternFileName(Pattern(), Event->Title(), Event->ShortText(), File());
+  isyslog("spawning timer %s for event %s", *ToDescr(), *Event->ToDescr());
+  cTimer *t = new cTimer(Event, FileName, this);
+  t->SetFlags(tfSpawned);
+  if (startswith(Pattern(), TIMERPATTERN_AVOID))
+     t->SetFlags(tfAvoid);
+  Timers->Add(t);
+  HandleRemoteTimerModifications(t);
+  return t;
+}
+
+bool cTimer::SpawnPatternTimers(const cSchedules *Schedules, cTimers *Timers)
+{
+  bool TimersSpawned = false;
   const cSchedule *Schedule = Schedules->GetSchedule(Channel());
   if (Schedule && Schedule->Events()->First()) {
-     if (Schedule->Modified(scheduleState)) {
+     if (Schedule->Modified(scheduleStateSpawn)) {
+        time_t Now = time(NULL);
+        // Find the first event that matches this pattern timer and either already has a spawned
+        // timer, or has not yet ended:
+        for (const cEvent *e = Schedule->Events()->First(); e; e = Schedule->Events()->Next(e)) {
+            if (Matches(e) != tmNone) {
+               const cTimer *Timer = Timers->GetTimerForEvent(e, tfSpawned); // a matching event that already has a spawned timer
+               if (!Timer && e->EndTime() > Now) { // only look at events that have not yet ended
+                  Timer = SpawnPatternTimer(e, Timers);
+                  TimersSpawned = true;
+                  }
+               if (Timer) {
+                  // Check all following matching events that would start while the first timer
+                  // is still recording:
+                  bool UseVps = Timer->HasFlags(tfVps);
+                  time_t Limit = Timer->StopTimeEvent();
+                  if (UseVps)
+                     Limit += EXPIRELATENCY;
+                  else
+                     Limit += Setup.MarginStart * 60;
+                  for (e = Schedule->Events()->Next(e); e; e = Schedule->Events()->Next(e)) {
+                      if (e->StartTime() <= Limit) {
+                         if (!Timers->GetTimerForEvent(e, tfSpawned) && Matches(e) != tmNone) {
+                            SpawnPatternTimer(e, Timers);
+                            TimersSpawned = true;
+                            }
+                         if (UseVps)
+                            break; // with VPS we only need to check the event immediately following the first one
+                         }
+                      else
+                         break; // no need to check events that are too far in the future
+                      }
+                  break;
+                  }
+               }
+            }
+        }
+     }
+  return TimersSpawned;
+}
+
+bool cTimer::AdjustSpawnedTimer(void)
+{
+  if (Event()) {
+     if (const cSchedule *Schedule = Event()->Schedule()) { // events may be deleted from their schedule in cSchedule::DropOutdated()!
+        if (Schedule->Modified(scheduleStateAdjust)) {
+           // Adjust the timer to shifted start/stop times of the event if necessary:
+           time_t tstart = Event()->StartTime();
+           time_t tstop = Event()->EndTime();
+           int MarginStart = 0;
+           int MarginStop  = 0;
+           CalcMargins(MarginStart, MarginStop, Event());
+           tstart -= MarginStart;
+           tstop  += MarginStop;
+           // Event start/end times are given in "seconds since the epoch". Some broadcasters use values
+           // that result in full minutes (with zero seconds), while others use any values. VDR's timers
+           // use times given in full minutes, truncating any seconds. Thus we only react if the start/stop
+           // times of the timer are off by at least one minute:
+           if (abs(StartTime() - tstart) >= 60 || abs(StopTime() - tstop) >= 60) {
+              cString OldDescr = ToDescr();
+              struct tm tm_r;
+              struct tm *time = localtime_r(&tstart, &tm_r);
+              SetDay(cTimer::SetTime(tstart, 0));
+              SetStart(time->tm_hour * 100 + time->tm_min);
+              time = localtime_r(&tstop, &tm_r);
+              SetStop(time->tm_hour * 100 + time->tm_min);
+              Matches();
+              isyslog("timer %s times changed to %s-%s", *OldDescr, *TimeString(tstart), *TimeString(tstop));
+              return true;
+              }
+           }
+        }
+     }
+  return false;
+}
+
+void cTimer::TriggerRespawn(void)
+{
+  if (Local() && HasFlags(tfSpawned) || IsPatternTimer()) {
+     if (Channel()) {
+        LOCK_CHANNELS_READ;
+        if (const cSchedule *Schedule = Channel()->Schedule()) {
+           dsyslog("triggering respawn for timer %s", *ToDescr());
+           LOCK_SCHEDULES_WRITE;
+           const_cast<cSchedule *>(Schedule)->SetModified();
+           }
+        }
+     }
+}
+
+bool cTimer::SetEventFromSchedule(const cSchedules *Schedules)
+{
+  if (IsPatternTimer())
+     return SetEvent(NULL);
+  const cSchedule *Schedule = Schedules->GetSchedule(Channel());
+  if (Schedule && Schedule->Events()->First()) {
+     if (Schedule->Modified(scheduleStateSet)) {
         const cEvent *Event = NULL;
         if (HasFlags(tfVps) && Schedule->Events()->First()->Vps()) {
            // VPS timers only match if their start time exactly matches the event's VPS time:
@@ -597,11 +911,11 @@ bool cTimer::SetEvent(const cEvent *Event)
      if (Event) {
         isyslog("timer %s set to event %s", *ToDescr(), *Event->ToDescr());
         Event->IncNumTimers();
-        Event->Schedule()->Modified(scheduleState); // to get the current state
+        Event->Schedule()->Modified(scheduleStateSet); // to get the current state
         }
      else {
         isyslog("timer %s set to no event", *ToDescr());
-        scheduleState = -1;
+        scheduleStateSet = scheduleStateSpawn = scheduleStateAdjust = -1;
         }
      event = Event;
      return true;
@@ -707,7 +1021,7 @@ void cTimer::Skip(void)
 
 void cTimer::OnOff(void)
 {
-  if (IsSingleEvent())
+  if (IsSingleEvent() || IsPatternTimer())
      InvFlags(tfActive);
   else if (day) {
      day = 0;
@@ -718,6 +1032,8 @@ void cTimer::OnOff(void)
   else
      SetFlags(tfActive);
   SetEvent(NULL);
+  if (HasFlags(tfActive))
+     TriggerRespawn(); // have pattern timers spawn if necessary
   Matches(); // refresh start and end time
 }
 
@@ -805,11 +1121,9 @@ const cTimer *cTimers::GetMatch(const cEvent *Event, eTimerMatch *Match) const
   eTimerMatch m = tmNone;
   for (const cTimer *ti = First(); ti; ti = Next(ti)) {
       eTimerMatch tm = ti->Matches(Event);
-      if (tm > m || tm == tmFull && ti->Local()) {
+      if (tm > m || tm == tmFull && t && (t->Remote() && ti->Local() || t->IsPatternTimer() && ti->HasFlags(tfSpawned))) {
          t = ti;
          m = tm;
-         if (m == tmFull && ti->Local())
-            break;
          }
       }
   if (Match)
@@ -817,6 +1131,17 @@ const cTimer *cTimers::GetMatch(const cEvent *Event, eTimerMatch *Match) const
   return t;
 }
 
+const cTimer *cTimers::GetTimerForEvent(const cEvent *Event, eTimerFlags Flags)
+{
+  if (Event && Event->HasTimer()) {
+     for (const cTimer *ti = First(); ti; ti = Next(ti)) {
+         if (ti->Event() == Event && ti->Local() && ti->HasFlags(Flags))
+            return ti;
+         }
+     }
+  return NULL;
+}
+
 int cTimers::GetMaxPriority(void) const
 {
   int n = -1;
@@ -831,7 +1156,7 @@ const cTimer *cTimers::GetNextActiveTimer(void) const
 {
   const cTimer *t0 = NULL;
   for (const cTimer *ti = First(); ti; ti = Next(ti)) {
-      if (!ti->Remote()) {
+      if (!ti->Remote() && !ti->IsPatternTimer()) {
          ti->Matches();
          if ((ti->HasFlags(tfActive)) && (!t0 || ti->StopTime() > time(NULL) && ti->Compare(*t0) < 0))
             t0 = ti;
@@ -882,20 +1207,50 @@ const cTimer *cTimers::UsesChannel(const cChannel *Channel) const
 bool cTimers::SetEvents(const cSchedules *Schedules)
 {
   bool TimersModified = false;
-  for (cTimer *ti = First(); ti; ti = Next(ti))
-      TimersModified |= ti->SetEventFromSchedule(Schedules);
+  for (cTimer *ti = First(); ti; ti = Next(ti)) {
+      if (!ti->IsPatternTimer())
+         TimersModified |= ti->SetEventFromSchedule(Schedules);
+      }
   return TimersModified;
 }
 
-bool cTimers::DeleteExpired(void)
+bool cTimers::SpawnPatternTimers(const cSchedules *Schedules)
 {
-  if (time(NULL) - lastDeleteExpired < 30)
+  bool TimersModified = false;
+  for (cTimer *ti = First(); ti; ti = Next(ti)) {
+      if (ti->IsPatternTimer() && ti->Local()) {
+         if (ti->HasFlags(tfActive))
+            TimersModified |= ti->SpawnPatternTimers(Schedules, this);
+         }
+      }
+  return TimersModified;
+}
+
+bool cTimers::AdjustSpawnedTimers(void)
+{
+  bool TimersModified = false;
+  for (cTimer *ti = First(); ti; ti = Next(ti)) {
+      if (ti->Local()) {
+         if (ti->HasFlags(tfSpawned) && !ti->HasFlags(tfVps))
+            TimersModified |= ti->AdjustSpawnedTimer();
+         }
+      }
+  return TimersModified;
+}
+
+#define DELETE_EXPIRED_TIMEOUT  30 // seconds
+
+bool cTimers::DeleteExpired(bool Force)
+{
+  if (!Force && time(NULL) - lastDeleteExpired < DELETE_EXPIRED_TIMEOUT)
      return false;
   bool TimersModified = false;
   cTimer *ti = First();
   while (ti) {
         cTimer *next = Next(ti);
         if (!ti->Remote() && ti->Expired()) {
+           ti->SetEvent(NULL); // Del() doesn't call ~cTimer() right away, so this is necessary here
+           ti->TriggerRespawn(); // in case this is a spawned timer
            isyslog("deleting timer %s", *ti->ToDescr());
            Del(ti);
            TimersModified = true;
