@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: timers.h 4.11.1.1 2019/05/23 10:02:45 kls Exp $
+ * $Id: timers.h 5.8 2021/04/20 13:22:37 kls Exp $
  */
 
 #ifndef __TIMERS_H
@@ -20,36 +20,44 @@ enum eTimerFlags { tfNone      = 0x0000,
                    tfInstant   = 0x0002,
                    tfVps       = 0x0004,
                    tfRecording = 0x0008,
+                   tfSpawned   = 0x0010,
+                   tfAvoid     = 0x0020,
                    tfAll       = 0xFFFF,
                  };
 enum eTimerMatch { tmNone, tmPartial, tmFull };
 
+class cTimers;
+
 class cTimer : public cListObject {
   friend class cMenuEditTimer;
 private:
   int id;
-  mutable time_t startTime, stopTime;
-  int scheduleState;
+  mutable time_t startTime, stopTime; ///< the time_t value calculated from 'day', 'start' and 'stop'
+  int scheduleStateSet;
+  int scheduleStateSpawn;
+  int scheduleStateAdjust;
   mutable time_t deferred; ///< Matches(time_t, ...) will return false if the current time is before this value
   bool pending, inVpsMargin;
   uint flags;
   const cChannel *channel;
   mutable time_t day; ///< midnight of the day this timer shall hit, or of the first day it shall hit in case of a repeating timer
   int weekdays;       ///< bitmask, lowest bits: SSFTWTM  (the 'M' is the LSB)
-  int start;
-  int stop;
+  int start;          ///< the start and stop time of this timer as given by the user,
+  int stop;           ///< in the form hhmm, with hh (00..23) and mm (00..59) added as hh*100+mm
   int priority;
   int lifetime;
+  mutable char pattern[NAME_MAX * 2 + 1]; // same size as 'file', to be able to initially fill 'pattern' with 'file' in the 'Edit timer' menu
   mutable char file[NAME_MAX * 2 + 1]; // *2 to be able to hold 'title' and 'episode', which can each be up to 255 characters long
   char *aux;
   char *remote;
   const cEvent *event;
 public:
   cTimer(bool Instant = false, bool Pause = false, const cChannel *Channel = NULL);
-  cTimer(const cEvent *Event);
+  cTimer(const cEvent *Event, const char *FileName = NULL, const cTimer *PatternTimer = NULL);
   cTimer(const cTimer &Timer);
   virtual ~cTimer();
   cTimer& operator= (const cTimer &Timer);
+  void CalcMargins(int &MarginStart, int &MarginStop, const cEvent *Event);
   virtual int Compare(const cListObject &ListObject) const;
   int Id(void) const { return id; }
   bool Recording(void) const { return HasFlags(tfRecording); }
@@ -63,12 +71,14 @@ public:
   int Stop(void) const { return stop; }
   int Priority(void) const { return priority; }
   int Lifetime(void) const { return lifetime; }
+  const char *Pattern(void) const { return pattern; }
   const char *File(void) const { return file; }
   time_t FirstDay(void) const { return weekdays ? day : 0; }
   const char *Aux(void) const { return aux; }
   const char *Remote(void) const { return remote; }
   bool Local(void) const { return !remote; } // convenience
   time_t Deferred(void) const { return deferred; }
+  cString PatternAndFile(void) const;
   cString ToText(bool UseChannelID = false) const;
   cString ToDescr(void) const;
   const cEvent *Event(void) const { return event; }
@@ -80,13 +90,21 @@ public:
   bool DayMatches(time_t t) const;
   static time_t IncDay(time_t t, int Days);
   static time_t SetTime(time_t t, int SecondsFromMidnight);
+  void SetPattern(const char *Pattern);
   void SetFile(const char *File);
+  bool IsPatternTimer(void) const { return *pattern; }
   bool Matches(time_t t = 0, bool Directly = false, int Margin = 0) const;
   eTimerMatch Matches(const cEvent *Event, int *Overlap = NULL) const;
   bool Expired(void) const;
-  time_t StartTime(void) const;
-  time_t StopTime(void) const;
+  time_t StartTime(void) const; ///< the start time as given by the user
+  time_t StopTime(void) const; ///< the stop time as given by the user
+  time_t StartTimeEvent(void) const; ///< the start/stop times as given by the event (for VPS timers), by event plus margins (for spawned non-VPS timers),
+  time_t StopTimeEvent(void) const; ///< or by the user (for normal timers)
   void SetId(int Id);
+  cTimer *SpawnPatternTimer(const cEvent *Event, cTimers *Timers);
+  bool SpawnPatternTimers(const cSchedules *Schedules, cTimers *Timers);
+  bool AdjustSpawnedTimer(void);
+  void TriggerRespawn(void);
   bool SetEventFromSchedule(const cSchedules *Schedules);
   bool SetEvent(const cEvent *Event);
   void SetRecording(bool Recording);
@@ -176,13 +194,16 @@ public:
   cTimer *GetMatch(time_t t) { return const_cast<cTimer *>(static_cast<const cTimers *>(this)->GetMatch(t)); };
   const cTimer *GetMatch(const cEvent *Event, eTimerMatch *Match = NULL) const;
   cTimer *GetMatch(const cEvent *Event, eTimerMatch *Match = NULL) { return const_cast<cTimer *>(static_cast<const cTimers *>(this)->GetMatch(Event, Match)); }
+  const cTimer *GetTimerForEvent(const cEvent *Event, eTimerFlags Flags = tfNone);
   int GetMaxPriority(void) const;
       ///< Returns the maximum priority of all local timers that are currently recording.
       ///< If there is no local timer currently recording, -1 is returned.
   const cTimer *GetNextActiveTimer(void) const;
   const cTimer *UsesChannel(const cChannel *Channel) const;
   bool SetEvents(const cSchedules *Schedules);
-  bool DeleteExpired(void);
+  bool SpawnPatternTimers(const cSchedules *Schedules);
+  bool AdjustSpawnedTimers(void);
+  bool DeleteExpired(bool Force);
   void Add(cTimer *Timer, cTimer *After = NULL);
   void Ins(cTimer *Timer, cTimer *Before = NULL);
   void Del(cTimer *Timer, bool DeleteObject = true);
