@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: tools.c 4.11 2018/03/04 10:28:04 kls Exp $
+ * $Id: tools.c 5.4 2021/05/20 10:13:43 kls Exp $
  */
 
 #include "tools.h"
@@ -151,6 +151,8 @@ char *strreplace(char *s, char c1, char c2)
 
 char *strreplace(char *s, const char *s1, const char *s2)
 {
+  if (!s || !s1 || !s2)
+     return s;
   char *p = strstr(s, s1);
   if (p) {
      int of = p - s;
@@ -198,6 +200,22 @@ int strcountchr(const char *s, char c)
   return n;
 }
 
+cString strgetbefore(const char *s, char c, int n)
+{
+  const char *p = strrchr(s, 0); // points to the terminating 0 of s
+  while (--p >= s) {
+        if (*p == c && --n == 0)
+           break;
+        }
+  return cString(s, p);
+}
+
+const char *strgetlast(const char *s, char c)
+{
+  const char *p = strrchr(s, c);
+  return p ? p + 1 : s;
+}
+
 char *stripspace(char *s)
 {
   if (s && *s) {
@@ -1099,12 +1117,16 @@ cString &cString::operator=(const char *String)
 
 cString &cString::Append(const char *String)
 {
-  int l1 = strlen(s);
-  int l2 = strlen(String);
-  char *p = (char *)realloc(s, l1 + l2 + 1);
-  if (p != s)
-     strcpy(p, s);
-  strcpy(p + l1, String);
+  if (String) {
+     int l1 = s ? strlen(s) : 0;
+     int l2 = strlen(String);
+     if (char *p = (char *)realloc(s, l1 + l2 + 1)) {
+        s = p;
+        strcpy(s + l1, String);
+        }
+     else
+        esyslog("ERROR: out of memory");
+     }
   return *this;
 }
 
@@ -1268,15 +1290,15 @@ static boolean JpegCompressEmptyOutputBuffer(j_compress_ptr cinfo)
         }
      else {
         esyslog("ERROR: out of memory");
-        return false;
+        return FALSE;
         }
      if (jcd->mem) {
         cinfo->dest->next_output_byte = jcd->mem + Used;
         cinfo->dest->free_in_buffer = jcd->size - Used;
-        return true;
+        return TRUE;
         }
      }
-  return false;
+  return FALSE;
 }
 
 static void JpegCompressTermDestination(j_compress_ptr cinfo)
@@ -1321,8 +1343,8 @@ uchar *RgbToJpeg(uchar *Mem, int Width, int Height, int &Size, int Quality)
   cinfo.in_color_space = JCS_RGB;
 
   jpeg_set_defaults(&cinfo);
-  jpeg_set_quality(&cinfo, Quality, true);
-  jpeg_start_compress(&cinfo, true);
+  jpeg_set_quality(&cinfo, Quality, TRUE);
+  jpeg_start_compress(&cinfo, TRUE);
 
   int rs = Width * 3;
   JSAMPROW rp[Height];
@@ -1611,8 +1633,10 @@ bool cFileNameList::Load(const char *Directory, bool DirsOnly)
 
 // --- cFile -----------------------------------------------------------------
 
+#if DEPRECATED_CFILE
 bool cFile::files[FD_SETSIZE] = { false };
 int cFile::maxFiles = 0;
+#endif
 
 cFile::cFile(void)
 {
@@ -1637,6 +1661,7 @@ bool cFile::Open(int FileDes)
   if (FileDes >= 0) {
      if (!IsOpen()) {
         f = FileDes;
+#if DEPRECATED_CFILE
         if (f >= 0) {
            if (f < FD_SETSIZE) {
               if (f >= maxFiles)
@@ -1650,27 +1675,31 @@ bool cFile::Open(int FileDes)
            else
               esyslog("ERROR: file descriptor %d is larger than FD_SETSIZE (%d)", f, FD_SETSIZE);
            }
+#endif
         }
      else
         esyslog("ERROR: attempt to re-open file descriptor %d", FileDes);
      }
-  return false;
+  return IsOpen();
 }
 
 void cFile::Close(void)
 {
   if (f >= 0) {
      close(f);
+#if DEPRECATED_CFILE
      files[f] = false;
+#endif
      f = -1;
      }
 }
 
 bool cFile::Ready(bool Wait)
 {
-  return f >= 0 && AnyFileReady(f, Wait ? 1000 : 0);
+  return f >= 0 && FileReady(f, Wait ? 1000 : 0);
 }
 
+#if DEPRECATED_CFILE
 bool cFile::AnyFileReady(int FileDes, int TimeoutMs)
 {
   fd_set set;
@@ -1688,6 +1717,7 @@ bool cFile::AnyFileReady(int FileDes, int TimeoutMs)
   timeout.tv_usec = (TimeoutMs % 1000) * 1000;
   return select(FD_SETSIZE, &set, NULL, NULL, &timeout) > 0 && (FileDes < 0 || FD_ISSET(FileDes, &set));
 }
+#endif
 
 bool cFile::FileReady(int FileDes, int TimeoutMs)
 {
@@ -1704,6 +1734,7 @@ bool cFile::FileReady(int FileDes, int TimeoutMs)
   return select(FD_SETSIZE, &set, NULL, NULL, (TimeoutMs >= 0) ? &timeout : NULL) > 0 && FD_ISSET(FileDes, &set);
 }
 
+#if DEPRECATED_CFILE
 bool cFile::FileReadyForWriting(int FileDes, int TimeoutMs)
 {
   fd_set set;
@@ -1716,6 +1747,7 @@ bool cFile::FileReadyForWriting(int FileDes, int TimeoutMs)
   timeout.tv_usec = TimeoutMs * 1000;
   return select(FD_SETSIZE, NULL, &set, NULL, &timeout) > 0 && FD_ISSET(FileDes, &set);
 }
+#endif
 
 // --- cSafeFile -------------------------------------------------------------
 
