@@ -1,7 +1,7 @@
 /*
  * vdr.c: Video Disk Recorder main program
  *
- * Copyright (C) 2000-2018 Klaus Schmidinger
+ * Copyright (C) 2000-2021 Klaus Schmidinger
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,7 @@
  *
  * The project's page is at http://www.tvdr.de
  *
- * $Id: vdr.c 4.25.1.5 2019/05/23 10:02:45 kls Exp $
+ * $Id: vdr.c 5.7 2021/12/27 13:31:04 kls Exp $
  */
 
 #include <getopt.h>
@@ -131,20 +131,18 @@ static bool DropCaps(void)
      fprintf(stderr, "vdr: cap_get_proc failed: %s\n", strerror(errno));
      return false;
      }
-  char *caps_text = cap_to_text(caps_all, NULL);
-  if (!caps_text) {
-     fprintf(stderr, "vdr: cap_to_text failed: %s\n", strerror(errno));
-     return false;
-     }
-  if (cap_free(caps_all)) {
-     fprintf(stderr, "vdr: cap_free failed: %s\n", strerror(errno));
+  cap_flag_value_t cap_flag_value;
+  if (cap_get_flag(caps_all, CAP_SYS_TIME, CAP_PERMITTED , &cap_flag_value)) {
+     fprintf(stderr, "vdr: cap_get_flag failed: %s\n", strerror(errno));
      return false;
      }
   cap_t caps;
-  if (strstr(caps_text,"cap_sys_time"))
+  if (cap_flag_value == CAP_SET)
      caps = cap_from_text("= cap_sys_nice,cap_sys_time,cap_net_raw=ep");
-  else
+  else {
+     fprintf(stdout,"vdr: OS does not support cap_sys_time\n");
      caps = cap_from_text("= cap_sys_nice,cap_net_raw=ep");
+     }
   if (!caps) {
      fprintf(stderr, "vdr: cap_from_text failed: %s\n", strerror(errno));
      return false;
@@ -239,13 +237,6 @@ int main(int argc, char *argv[])
   int WatchdogTimeout = DEFAULTWATCHDOG;
   const char *Terminal = NULL;
   const char *OverrideCharacterTable = NULL;
-#ifndef DEPRECATED_VDR_CHARSET_OVERRIDE
-#define DEPRECATED_VDR_CHARSET_OVERRIDE 0
-#endif
-#if DEPRECATED_VDR_CHARSET_OVERRIDE
-  OverrideCharacterTable = getenv("VDR_CHARSET_OVERRIDE");
-  const char *DeprecatedVdrCharsetOverride = OverrideCharacterTable;
-#endif
 
   bool UseKbd = true;
   const char *LircDevice = NULL;
@@ -424,6 +415,7 @@ int main(int argc, char *argv[])
                           SysLogLevel = l;
                           if (!p)
                              break;
+                          *p = '.';
                           if (isnumber(p + 1)) {
                              int l = atoi(p + 1);
                              if (0 <= l && l <= 7) {
@@ -723,13 +715,9 @@ int main(int argc, char *argv[])
      isyslog("codeset is '%s' - %s", CodeSet, known ? "known" : "unknown");
      cCharSetConv::SetSystemCharacterTable(CodeSet);
      }
-#if DEPRECATED_VDR_CHARSET_OVERRIDE
-  if (DeprecatedVdrCharsetOverride)
-     isyslog("use of environment variable VDR_CHARSET_OVERRIDE (%s) is deprecated!", DeprecatedVdrCharsetOverride);
-#endif
   if (OverrideCharacterTable) {
-     isyslog("override character table is '%s'", OverrideCharacterTable);
-     SI::SetOverrideCharacterTable(OverrideCharacterTable);
+     bool known = SI::SetOverrideCharacterTable(OverrideCharacterTable);
+     isyslog("override character table is '%s' - %s", OverrideCharacterTable, known ? "known" : "unknown");
      }
 
   // Initialize internationalization:
@@ -785,6 +773,7 @@ int main(int argc, char *argv[])
   KeyMacros.Load(AddDirectory(ConfigDirectory, "keymacros.conf"), true);
   Folders.Load(AddDirectory(ConfigDirectory, "folders.conf"));
   CamResponsesLoad(AddDirectory(ConfigDirectory, "camresponses.conf"), true);
+  DoneRecordingsPattern.Load(AddDirectory(CacheDirectory, "donerecs.data"));
 
   if (!*cFont::GetFontFileName(Setup.FontOsd)) {
      const char *msg = "no fonts available - OSD will not show any text!";
@@ -1099,15 +1088,24 @@ int main(int argc, char *argv[])
           static cStateKey TimersStateKey;
           cTimers *Timers = cTimers::GetTimersWrite(TimersStateKey);
           {
+            LOCK_CHANNELS_READ; // Channels are needed for spawning pattern timers!
             // Assign events to timers:
             static cStateKey SchedulesStateKey;
             if (TimersStateKey.StateChanged())
                SchedulesStateKey.Reset(); // we assign events if either the Timers or the Schedules have changed
             bool TimersModified = false;
             if (const cSchedules *Schedules = cSchedules::GetSchedulesRead(SchedulesStateKey)) {
-               Timers->SetSyncStateKey(StateKeySVDRPRemoteTimersPoll);
+               Timers->SetSyncStateKey(StateKeySVDRPRemoteTimersPoll); // setting events shall not trigger a remote timer poll...
+               if (Timers->AdjustSpawnedTimers()) { // must do this *before* SetEvents()!
+                  StateKeySVDRPRemoteTimersPoll.Reset(); // ...but adjusting spawned timers...
+                  TimersModified = true;
+                  }
                if (Timers->SetEvents(Schedules))
                   TimersModified = true;
+               if (Timers->SpawnPatternTimers(Schedules)) {
+                  StateKeySVDRPRemoteTimersPoll.Reset(); // ...or spawning new timers must!
+                  TimersModified = true;
+                  }
                SchedulesStateKey.Remove();
                }
             TimersStateKey.Remove(TimersModified); // we need to remove the key here, so that syncing StateKeySVDRPRemoteTimersPoll takes effect!
@@ -1131,7 +1129,7 @@ int main(int argc, char *argv[])
           if (Now - LastTimerCheck > TIMERCHECKDELTA) { // don't do this too often
              InhibitEpgScan = false;
              for (cTimer *Timer = Timers->First(); Timer; Timer = Timers->Next(Timer)) {
-                 if (Timer->Remote())
+                 if (Timer->Remote() || Timer->IsPatternTimer())
                     continue;
                  bool InVpsMargin = false;
                  bool NeedsTransponder = false;
@@ -1179,7 +1177,7 @@ int main(int argc, char *argv[])
              LastTimerCheck = Now;
              }
           // Delete expired timers:
-          if (Timers->DeleteExpired())
+          if (Timers->DeleteExpired(TimersModified))
              TimersModified = true;
           // Make sure there is enough free disk space for ongoing recordings:
           int MaxPriority = Timers->GetMaxPriority();
@@ -1198,8 +1196,19 @@ int main(int argc, char *argv[])
         // Queued messages:
         Skins.ProcessQueuedMessages();
         // User Input:
-        cOsdObject *Interact = Menu ? Menu : cControl::Control();
-        eKeys key = Interface->GetKey(!Interact || !Interact->NeedsFastResponse());
+        bool NeedsFastResponse = Menu && Menu->NeedsFastResponse();
+        if (!NeedsFastResponse) {
+           // Must limit the scope of ControlMutexLock here to not hold the lock during the call to Interface->GetKey().
+           cMutexLock ControlMutexLock;
+           cControl *Control = cControl::Control(ControlMutexLock);
+           NeedsFastResponse = Control && Control->NeedsFastResponse();
+           }
+        eKeys key = Interface->GetKey(!NeedsFastResponse);
+        cOsdObject *Interact = Menu;
+        cMutexLock ControlMutexLock;
+        cControl *Control = NULL;
+        if (!Menu)
+           Interact = Control = cControl::Control(ControlMutexLock);
         if (ISREALKEY(key)) {
            EITScanner.Activity();
            // Cancel shutdown countdown:
@@ -1217,9 +1226,9 @@ int main(int argc, char *argv[])
                bool WasMenu = Interact && Interact->IsMenu();
                if (Menu)
                   DELETE_MENU;
-               else if (cControl::Control()) {
+               else if (Control) {
                   if (cOsd::IsOpen())
-                     cControl::Control()->Hide();
+                     Control->Hide();
                   else
                      WasOpen = false;
                   }
@@ -1235,9 +1244,9 @@ int main(int argc, char *argv[])
                   }
                else if (!Menu) {
                   IsInfoMenu = true;
-                  if (cControl::Control()) {
-                     cControl::Control()->Hide();
-                     Menu = cControl::Control()->GetInfo();
+                  if (Control) {
+                     Control->Hide();
+                     Menu = Control->GetInfo();
                      if (Menu)
                         Menu->Show();
                      else
@@ -1254,8 +1263,8 @@ int main(int argc, char *argv[])
           // Direct main menu functions:
           #define DirectMainFunction(function)\
             { DELETE_MENU;\
-            if (cControl::Control())\
-               cControl::Control()->Hide();\
+            if (Control)\
+               Control->Hide();\
             Menu = new cMenuMain(function);\
             key = kNone; } // nobody else needs to see this key
           case kSchedule:   DirectMainFunction(osSchedule); break;
@@ -1269,8 +1278,8 @@ int main(int argc, char *argv[])
                const char *PluginName = cRemote::GetPlugin();
                if (PluginName) {
                   DELETE_MENU;
-                  if (cControl::Control())
-                     cControl::Control()->Hide();
+                  if (Control)
+                     Control->Hide();
                   cPlugin *plugin = cPluginManager::GetPlugin(PluginName);
                   if (plugin) {
                      Menu = plugin->MainMenuAction();
@@ -1292,7 +1301,7 @@ int main(int argc, char *argv[])
                   Menu = new cDisplayChannel(NORMALKEY(key));
                   continue;
                   }
-               else if (cDisplayChannel::IsOpen() || cControl::Control()) {
+               else if (cDisplayChannel::IsOpen() || Control) {
                   Interact->ProcessKey(key);
                   continue;
                   }
@@ -1320,8 +1329,8 @@ int main(int argc, char *argv[])
                break;
           // Audio track control:
           case kAudio:
-               if (cControl::Control())
-                  cControl::Control()->Hide();
+               if (Control)
+                  Control->Hide();
                if (!cDisplayTracks::IsOpen()) {
                   DELETE_MENU;
                   Menu = cDisplayTracks::Create();
@@ -1332,8 +1341,8 @@ int main(int argc, char *argv[])
                break;
           // Subtitle track control:
           case kSubtitles:
-               if (cControl::Control())
-                  cControl::Control()->Hide();
+               if (Control)
+                  Control->Hide();
                if (!cDisplaySubtitleTracks::IsOpen()) {
                   DELETE_MENU;
                   Menu = cDisplaySubtitleTracks::Create();
@@ -1345,7 +1354,7 @@ int main(int argc, char *argv[])
           // Pausing live video:
           case kPlayPause:
           case kPause:
-               if (!cControl::Control()) {
+               if (!Control) {
                   DELETE_MENU;
                   if (Setup.PauseKeyHandling) {
                      if (Setup.PauseKeyHandling > 1 || Interface->Confirm(tr("Pause live video?"))) {
@@ -1358,7 +1367,7 @@ int main(int argc, char *argv[])
                break;
           // Instant recording:
           case kRecord:
-               if (!cControl::Control()) {
+               if (!Control) {
                   if (Setup.RecordKeyHandling) {
                      if (Setup.RecordKeyHandling > 1 || Interface->Confirm(tr("Start recording?"))) {
                         if (cRecordControls::Start())
@@ -1397,15 +1406,16 @@ int main(int argc, char *argv[])
                break;
           default: break;
           }
-        Interact = Menu ? Menu : cControl::Control(); // might have been closed in the mean time
+        Interact = Menu ? Menu : Control; // might have been closed in the mean time
         if (Interact) {
            LastInteract = Now;
            eOSState state = Interact->ProcessKey(key);
-           if (state == osUnknown && Interact != cControl::Control()) {
-              if (ISMODELESSKEY(key) && cControl::Control()) {
-                 state = cControl::Control()->ProcessKey(key);
+           if (state == osUnknown && Interact != Control) {
+              if (ISMODELESSKEY(key) && Control) {
+                 state = Control->ProcessKey(key);
                  if (state == osEnd) {
                     // let's not close a menu when replay ends:
+                    Control = NULL;
                     cControl::Shutdown();
                     continue;
                     }
@@ -1424,15 +1434,18 @@ int main(int argc, char *argv[])
                             break;
              case osRecordings:
                             DELETE_MENU;
+                            Control = NULL;
                             cControl::Shutdown();
                             Menu = new cMenuMain(osRecordings, true);
                             break;
              case osReplay: DELETE_MENU;
+                            Control = NULL;
                             cControl::Shutdown();
                             cControl::Launch(new cReplayControl);
                             break;
              case osStopReplay:
                             DELETE_MENU;
+                            Control = NULL;
                             cControl::Shutdown();
                             break;
              case osPlugin: DELETE_MENU;
@@ -1443,8 +1456,10 @@ int main(int argc, char *argv[])
              case osBack:
              case osEnd:    if (Interact == Menu)
                                DELETE_MENU;
-                            else
+                            else {
+                               Control = NULL;
                                cControl::Shutdown();
+                               }
                             break;
              default:       ;
              }
@@ -1489,6 +1504,7 @@ int main(int argc, char *argv[])
              // Instant resume of the last viewed recording:
              case kPlay:
                   if (cReplayControl::LastReplayed()) {
+                     Control = NULL;
                      cControl::Shutdown();
                      cControl::Launch(new cReplayControl);
                      }
@@ -1514,6 +1530,7 @@ int main(int argc, char *argv[])
         int NewPrimaryDVB = Setup.PrimaryDVB;
         if (NewPrimaryDVB != OldPrimaryDVB) {
            DELETE_MENU;
+           Control = NULL;
            cControl::Shutdown();
            Skins.QueueMessage(mtInfo, tr("Switching primary DVB..."));
            cOsdProvider::Shutdown();
@@ -1534,7 +1551,7 @@ int main(int argc, char *argv[])
               ShutdownHandler.countdown.Cancel();
            }
 
-        if (!cControl::Control() && !cRecordControls::Active() && !RecordingsHandler.Active() && (Now - cRemote::LastActivity()) > ACTIVITYTIMEOUT) {
+        if (!Control && !cRecordControls::Active() && !RecordingsHandler.Active() && (Now - cRemote::LastActivity()) > ACTIVITYTIMEOUT) {
            // Shutdown:
            // Check whether VDR will be ready for shutdown in SHUTDOWNWAIT seconds:
            time_t Soon = Now + SHUTDOWNWAIT;
